#include <avr/io.h>
#include <avr/interrupt.h>
#include "HD44780.h"

#define LCD_SetLed()    PORTC |= 0x04; // nastaveni podsviceni LCD
#define LCD_ClrLed()    PORTC &=~0x04; // nulovani podsviceni LCD
#define DS1820_Set()    PORTD |= 0x10; // nastaveni bitu teplomeru
#define DS1820_Clr()    PORTD &=~0x10; // nulovani bitu teplomeru
#define DS1302_SetRst() PORTC |= 0x08; // nastaveni DS1302 reset
#define DS1302_ClrRst() PORTC &=~0x08; // nulovani DS1302 reset
#define DS1302_SetSda() PORTC |= 0x10; // nastaveni DS1302 data
#define DS1302_ClrSda() PORTC &=~0x10; // nulovani DS1302 data
#define DS1302_OutSda() DDRC  |= 0x10; // vystup dat na DS1302
#define DS1302_InpSda() DDRC  &=~0x10; // vstup dat na DS1302
#define DS1302_SetClk() PORTC |= 0x20; // nastaveni DS1302 hodiny
#define DS1302_ClrClk() PORTC &=~0x20; // nulovani DS1302 hodiny
#define DS18_Set() PORTD |= 0x10; // pripojeni obvodu DS18B20
#define DS18_Clr() PORTD &=~0x10;
#define DS18_Out() DDRD  |= 0x10;
#define DS18_In()  DDRD  &=~0x10;

unsigned char DS_Reset(void); // Reset obvodu Dallas
unsigned char DS_Read(void); // Cte bajt z obvodu Dallas
void DS_Write( unsigned char value ); // Zapis bajtu na sbernici 1-wire
void EEPROM_Write(unsigned char addr, unsigned char val); // zapis do pameti EEPROM
unsigned char EEPROM_Read(unsigned char val); // cteni z pameti EEPROM
void SYS_Init(void); // inicializace procesoru, rozsviti LED
unsigned char DS1302( unsigned char DS1302addr, unsigned char DS1302data ); // ovladani obvodu DS1302
void SET_Rele(void); // nastaveni Rele
void SET_Cas(void); // nastaveni casu v EEPROM
void SET_Hodiny(void); // nastaveni noveho casu
void SET_Triak(void); // nastaveni triaku podle hodnoty Rele
void Search_Timer(void); // nastaveni rele podle casu v EEPROM
void Print_Stav( unsigned char val ); // napise mezeru a stav rele
void SET_Temp(void); // nastavuje teplotu termostatu

volatile unsigned int KEYmin, KEYmax, KEYcounter; // promenne pocitadla
volatile unsigned char KEYtimer1, KEYtimer, KEYenter, KEYenter1, KEYhex; // promenne pro praci klavesnice
volatile unsigned char Rsec, Rmin, Rhod; // promenne realneho casu
volatile unsigned char Ssec, Smin, Shod; // promenne casu k nastaveni
volatile unsigned char Rele; // bity kopirovane na triaky
volatile unsigned char Timer0, LedTimer; // casovac na pul sekundy
volatile unsigned char Tcounter; // ukazuje na misto v EEPROM kde se bude priste delat zmena
volatile unsigned char SetPWM; // ukazuje stav PWM
volatile unsigned int Teplota, Termostat;

int main(void)
{
  unsigned int val;
  unsigned char i;

  SYS_Init(); // inicializace
  while(1)
  {
    LCD_Position( 0, 0 );
	LCD_PrHex( Rhod ); // napise hodiny
	LCD_WriteData( ':' );
	LCD_PrHex( Rmin ); // napise minuty
	LCD_WriteData( ':' );
	LCD_PrHex( Rsec ); // napise sekundy
	LCD_WriteData( ' ' );

	val = Teplota;
	if(Teplota==0xFFFF)
	{
	  LCD_WriteCString(" ERROR ");
	}
	else // bude vypisovat teplotu
	{
 	  if (val&0x0800)
  	   {
         LCD_WriteData('-');
	     val = ~val+1;
  	   }
	  i = (unsigned char) (val>>4); // cela cast
	  val = (val&0x000F) * 625; // desetinna cast
	  i = i % 100;
      LCD_WriteData((i / 10) + 0x30); // znak 1
	  LCD_WriteData((i % 10) + 0x30); // znak 2
	  LCD_WriteData(','); // znak 3
      LCD_WriteData((val / 1000) + 0x30); // znak 4
	  val = val % 1000;
      LCD_WriteData((val / 100) + 0x30); // znak 5
	  LCD_WriteData( 0xDF ); // znak stupen 6
	  LCD_WriteCString( "C   " ); // znak 7
	}
    LCD_Position( 1, 0 );
	LCD_PrDec( Tcounter );
	LCD_WriteData( '>' );
    LCD_PrHex( EEPROM_Read( Tcounter * 4 + 0 ));
    LCD_WriteData( ':' );
    LCD_PrHex( EEPROM_Read( Tcounter * 4 + 1 ));
    LCD_WriteData( ':' );
    LCD_PrHex( EEPROM_Read( Tcounter * 4 + 2 ));
	Print_Stav( EEPROM_Read( Tcounter * 4 + 3 ));
	if(KEYenter==2)
	{
	  KEYmin = 0;
	  KEYmax = 3; // nastaveni 4 veci
	  KEYcounter = 0;
	  KEYenter = 0;
	  LCD_Position( 0, 0 );
	  LCD_WriteCString("Nastaveni:      ");
	  LCD_Position( 1, 0 );
	  LCD_WriteCString("Rel-Cas-Hod-Tep ");
	  LCD_WriteComm( 0b00001111 ); // nastavi blikajici kurzor
	  while(KEYenter==0)
	  {
		switch(KEYcounter)
		{
		  case 0: LCD_Position( 1, 0  ); break;
		  case 1: LCD_Position( 1, 4  ); break;
		  case 2: LCD_Position( 1, 8  ); break;
		  case 3: LCD_Position( 1, 12 ); break;
		}
	  }
	  if(KEYenter==1)
	  {
		KEYenter=0;
		WDTCR = 0x1F;
		WDTCR = 0x07; // deaktivuje Watchdog
		switch(KEYcounter)
		{
		  case 0: SET_Rele(); break;
		  case 1: SET_Cas(); break;
		  case 2: SET_Hodiny(); break;
		  case 3: SET_Temp(); break;
		}
		asm("wdr"); // nuluje watchdog
		WDTCR = 0x0F; // aktivuje Watchdog
	  }
	  LCD_WriteComm( 0b00001100 ); // vypne blikajici kurzor
	}
	KEYenter = 0;
  }
}

void SET_Temp(void) // nastavuje teplotu termostatu **************************************
{
  unsigned int val;
  unsigned char i;
  
  asm("wdr"); // nuluje watchdog
  WDTCR = 0x0F; // aktivuje Watchdog
  LCD_Init();
  LCD_WriteComm( 0b00001111 ); // nastavi blikajici kurzor
  LCD_Position( 0, 0 );
  LCD_WriteCString("Termostat:      ");
  KEYmin = 0;
  KEYmax = 1000; // maximum teploty 60 stupnu
  KEYcounter = Termostat;
  while( KEYenter!=2 )
  {
	LCD_Position( 1, 0 );
	val = KEYcounter;
	Termostat = KEYcounter;
 	if (val&0x0800)
  	 {
       LCD_WriteData('-');
	   val = ~val+1;
  	 }
	i = (unsigned char) (val>>4); // cela cast
	val = (val&0x000F) * 625; // desetinna cast
	i = i % 100;
    LCD_WriteData((i / 10) + 0x30);
	LCD_WriteData((i % 10) + 0x30);
	LCD_WriteData(',');
    LCD_WriteData((val / 1000) + 0x30);
	val = val % 1000;
    LCD_WriteData((val / 100) + 0x30);
	val = val % 100;
	LCD_WriteData((val / 10) + 0x30);
	LCD_WriteData((val % 10) + 0x30);
	LCD_WriteData( ' ' );
	LCD_WriteData( 0xDF ); // znak stupen
	LCD_WriteCString( "C   " );
	Wait_ms( 100 ); 
  }
  KEYenter = 0;
  EEPROM_Write( 0x80, (Termostat / 0x100));
  EEPROM_Write( 0x81, (Termostat % 0x100));
}

void Print_Stav( unsigned char val ) // napise mezeru a stav rele
{
  LCD_WriteData( ' ' );
  LCD_WriteData( 'R' );
  LCD_PrHex1((val&0x0F) + 1 ); // cislo rele
  switch(val&0xF0)
  {
	case 0x00: LCD_WriteCString(" _"); break;
	case 0x10: LCD_WriteCString(" ^"); break;
	default:   LCD_WriteCString(" x"); break;
  }
}

void Search_Timer(void) // nastaveni rele podle casu v EEPROM
{
  unsigned char h, m, s, i;
  unsigned long int Rtime, Etime, Ttime = 0xFFFFFF;

  Rtime = 0;
  Rtime = Rsec + ( Rmin * 256 ) + ( Rhod * 65536 ); // realny cas ve forme long int
  h = EEPROM_Read( Tcounter * 4 + 0 );
  m = EEPROM_Read( Tcounter * 4 + 1 );
  s = EEPROM_Read( Tcounter * 4 + 2 );
  Etime = 0;
  Etime = s + ( m * 256 ) + ( h * 65536 );
  if(Rtime==Etime) // nastavi rele podle noveho udaje
  {
    i = EEPROM_Read( Tcounter * 4 + 3 );
	switch(i&0x30)
	{
	  case 0x00: // vypinani rele
	  {
	    switch(i&0x03)
		{
		  case 0x00:
		  { 
		    Rele &= 0b11111110;
			SetPWM |= 0b00000101;
			SetPWM &= 0b11111101; 
			break;
		  }
		  case 0x01:
		  { 
		    Rele &= 0b11111101;
		    SetPWM |= 0b01010000;
			SetPWM &= 0b11011111;
			break;
		  }
		  case 0x02:
		  {
		    Rele &= 0b11111011;
		    SetPWM |= 0b01010000;
			SetPWM &= 0b11011111;
			break;
		  }
		  case 0x03: Rele &= 0b11110111; break;
		}
		break;
	  }
	  case 0x10: // zapinani rele
	  {
	    switch(i&0x03)
		{
		  case 0x00:
		  { 
		    Rele |= 0b00000001;
			SetPWM |= 0b00000111;
			break;
		  }
		  case 0x01:
		  {
		    Rele |= 0b00000010; 
			SetPWM |= 0b01110000;
			break;
		  }
		  case 0x02:
		  {
		    Rele |= 0b00000100;
			SetPWM |= 0b01110000;
			break;
		  }
		  case 0x03: Rele |= 0b00001000; break;
		}
		break;
	  }
	}
	LCD_SetLed();
	LedTimer = 0x60; // casovac nastaven
  }
  
  Ttime = 0xFFFFFF;
  for(i=1; i<21; i++) // hledani nejblizsiho casu
  {
    h = EEPROM_Read( i * 4 + 0 );
    m = EEPROM_Read( i * 4 + 1 );
    s = EEPROM_Read( i * 4 + 2 );
	Etime = 0;
	Etime = s + ( m * 256 ) + ( h * 65536 );
	if(( Rtime <= Etime ) && ( Etime <= Ttime )) 
	{
	  Ttime = Etime;
	  Tcounter = i; // nasel novy cas ve stejnem dni
	}
  }
}

void SET_Triak(void) // nastaveni triaku podle hodnoty Rele
{
  unsigned int val;  

  if((SetPWM&0x03)==0x03) // PWM A jede nahoru
  {
	val = OCR1A;
	if(val >= 4000)
	{
	  val--;
	  OCR1A = val;
	}
	else
	{
	  SetPWM &=~0x01; // nuluje bit
	  OCR1A = 10016;
	}
	if(val == 4100) PORTB |= 0b00000001; // rozsviti zarivky
  }

  if((SetPWM&0x03)==0x01) // PWM A jede dolu
  {
    if((SetPWM&0x08)==0x08)
	{
	  val = OCR1A;
	  if(val<10000)
	  {
		val++;
		OCR1A = val;
	  }
	  else
	  {
	  	OCR1A = 10016;
		SetPWM &=~0x09; // zastavuje chod
	  }
	}
	else
	{
	  SetPWM |= 0x08; // spousti
  	  OCR1A  = 4000;
	  PORTB &= 0b11111110; // zhasne zarivky
	}
  }
  if((SetPWM&0x30)==0x30) // PWM B jede nahoru
  {
	val = OCR1B;
	if(val >= 4000)
	{
	  val--;
	  OCR1B = val;
	}
	else
	{
	  SetPWM &=~0x10; // nuluje bit
	  OCR1B = 10016;
	}
	if(val == 4100)
	{
	  if((Rele&0x02)==0x02) PORTD |= 0b00100000;
	  if((Rele&0x04)==0x04) PORTD |= 0b10000000; // rozsviti zarivky	
	}
  }

  if((SetPWM&0x30)==0x10) // PWM B jede dolu
  {
    if((SetPWM&0x80)==0x80)
	{
	  val = OCR1B;
	  if(val<10000)
	  {
		val++;
		OCR1B = val;
	  }
	  else
	  {
	  	OCR1B = 10016;
		SetPWM &=~0x90; // zastavuje chod
	  }
	}
	else
	{
	  SetPWM |= 0x80; // spousti
  	  OCR1B  = 4000;
	  if((Rele&0x02)==0) PORTD &= 0b11011111; // zhasina zarivky
	  if((Rele&0x04)==0) PORTD &= 0b01111111;
	}
  }

  if((Rele&0x08)==0) PORTD &= 0b10111111;
  else PORTD |= 0b01000000;
}

void SET_Hodiny(void) // nastaveni noveho casu
{
  unsigned char i = 0;

  asm("wdr"); // nuluje watchdog
  WDTCR = 0x0F; // aktivuje Watchdog
  KEYhex = 0; // decimalni pocitani
  LCD_Init();
  LCD_WriteComm( 0b00001111 ); // nastavi blikajici kurzor
  LCD_Position( 0, 0 );
  LCD_WriteCString("Nastaveni hodin:");
  Ssec = Rsec; Smin = Rmin; Shod = Rhod; // nastaveny hodnoty ke zmene
  KEYcounter = 0;
  while( KEYenter!=2 )
  {
	KEYmin = 0;
    KEYmax = 2; // nastaveni hodin
	LCD_Position( 1, 0 );
    LCD_PrHex( Shod );
    LCD_WriteData( ':' );
    LCD_PrHex( Smin );
    LCD_WriteData( ':' );
    LCD_PrHex( Ssec );
	LCD_Position( 1, KEYcounter * 3 );
	Wait_ms( 300 );
	if( KEYenter==1 )
	{
	  KEYenter = 0;
	  i = KEYcounter;
	  switch(KEYcounter)
	  {
	    case 0: KEYcounter = Shod; KEYmax = 0x23; break;
		case 1: KEYcounter = Smin; KEYmax = 0x59; break;
		case 2: KEYcounter = Ssec; KEYmax = 0x59; break;
	  }
	  LCD_WriteComm( 0b00001110 ); // vypne blikajici kurzor
	  while( KEYenter==0 )
	  {
		LCD_Position( 1, i * 3 );
		LCD_PrHex( KEYcounter );
		LCD_Position( 1, i * 3 + 1 );
	    switch( i )
		{
		  case 0: Shod = KEYcounter; break;
		  case 1: Smin = KEYcounter; break;
		  case 2: Ssec = KEYcounter; break;
		}
		Wait_ms( 100 );
	  }
	  KEYenter = 0;
	  KEYcounter = i;
	  LCD_WriteComm( 0b00001111 ); // nastavi blikajici kurzor
	}
  }
  KEYenter = 0;
  KEYhex = 1; // vypne decimalni pocitani
  DS1302( 0x04, Shod);
  DS1302( 0x02, Smin);
  DS1302( 0x00, Ssec); // ulozi zmenene hodnoty
}

void SET_Cas(void) // nastaveni casu v EEPROM
{
  unsigned char i = 0, c = 0, Stav, Stav0 = 0, Stav1 = 0;

  asm("wdr"); // nuluje watchdog
  WDTCR = 0x0F; // aktivuje Watchdog
  LCD_Init();
  LCD_Position( 0, 0 );
  LCD_WriteCString("Nastaveni casu: ");
  KEYcounter = 1;
  while( KEYenter!=2 )
  {
	KEYmin = 1;
    KEYmax = 20; // nastaveni casu
	Shod = EEPROM_Read( KEYcounter * 4 + 0 );
	if(Shod>0x23) Shod = 0x12;
	Smin = EEPROM_Read( KEYcounter * 4 + 1 );
	if(Smin>0x59) Shod = 0x30;
	Ssec = EEPROM_Read( KEYcounter * 4 + 2 );
	if(Smin>0x59) Shod = 0x30;
	Stav = EEPROM_Read( KEYcounter * 4 + 3 );
	LCD_Position( 1, 0 );
	LCD_PrDec( KEYcounter );
	LCD_WriteData( '>' );
    LCD_PrHex( Shod );
    LCD_WriteData( ':' );
    LCD_PrHex( Smin );
    LCD_WriteData( ':' );
    LCD_PrHex( Ssec );
	Print_Stav( Stav );
	Stav0 = Stav & 0x03;
	Stav1 = (Stav >> 4) & 0x03;
	if( KEYenter==1 )
	{
	  KEYenter = 0;
	  c = KEYcounter; // ulozi hodnotu v EEPROM
	  KEYcounter = 0;
	  LCD_WriteComm( 0b00001111 ); // nastavi blikajici kurzor
	  while( KEYenter!=2 )
	  { // smycka nastaveni hodnoty co se bude menit
		KEYmin = 0;
		KEYmax = 4;
		LCD_Position( 1, KEYcounter * 3 + 3 );
		Wait_ms( 100 );
		if( KEYenter==1 )
		{ // volba nastaveni hodnoty pro zmenu sec, min, hod, stav
		  KEYenter = 0;
		  KEYhex = 0; // zapne decimalni pocitani
		  i = KEYcounter; // ulozi hodnotu ktera se ma menit
		  switch(i)
		  {
	    	case 0: KEYcounter = Shod;  KEYmax = 0x23; break;
			case 1: KEYcounter = Smin;  KEYmax = 0x59; break;
			case 2: KEYcounter = Ssec;  KEYmax = 0x59; break;
			case 3: KEYcounter = Stav0; KEYmax = 0x03; break; // zmena rele
			case 4: KEYcounter = Stav1; KEYmax = 0x02; break; // zmena stavu rele
		  }
		  LCD_WriteComm( 0b00001110 ); // vypne blikajici kurzor
		  while( KEYenter==0 ) // bude se menit hodnota v EEPROM
		  { // smycka nastaveni hodnoty pro zmenu sec, min, hod, stav
			LCD_Position( 1, i * 3 + 3 );
			switch(i)
			{
			  case 4:
			  {
			    LCD_Position( 1, 14 );
				switch( KEYcounter )
			    {
				  case 0:  LCD_WriteCString(" _"); break;
				  case 1:  LCD_WriteCString(" ^"); break;
				  default: LCD_WriteCString(" x"); break;
			    }
			    LCD_Position( 1, 15 );
				break;
			  }
			  case 3:
			  {
			    LCD_WriteData( 'R' );
				LCD_PrHex1( KEYcounter + 1 );
			    LCD_Position( 1, 12 );
			    break;
			  }
			  default:
			  {
			    LCD_PrHex( KEYcounter );
			    LCD_Position( 1, i * 3 + 4 );
				break;
			  }
			}
			switch( i )
			{
		  	  case 0: Shod  = KEYcounter; break;
		  	  case 1: Smin  = KEYcounter; break;
		  	  case 2: Ssec  = KEYcounter; break;
			  case 3: Stav0 = KEYcounter; break;
			  case 4: Stav1 = KEYcounter; break;
			}
			Stav = 0;
			Stav = Stav0 | ( Stav1 << 4 );
			Wait_ms( 100 );
		  } // smycka nastaveni hodnoty pro zmenu sec, min, hod, stav
	      KEYenter = 0;
	      KEYcounter = i;
	      LCD_WriteComm( 0b00001111 ); // nastavi blikajici kurzor
		  KEYhex = 1; // vypne decimalni pocitani
		} // volba nastaveni hodnoty pro zmenu sec, min, hod, stav
	  } // smycka nastaveni hodnoty co se bude menit
	  KEYenter = 0;	  
	  KEYcounter = c; // obnovi hodnotu pro EEPROM
	  EEPROM_Write( KEYcounter * 4 + 0, Shod );
	  EEPROM_Write( KEYcounter * 4 + 1, Smin );
	  EEPROM_Write( KEYcounter * 4 + 2, Ssec );
	  EEPROM_Write( KEYcounter * 4 + 3, Stav );
	  LCD_WriteComm( 0b00001100 ); // vypne blikajici kurzor
	}
  }
  KEYenter = 0;
}

void SET_Rele(void) // nastaveni Rele
{
  unsigned char i;

  asm("wdr"); // nuluje watchdog
  WDTCR = 0x0F; // aktivuje Watchdog
  LCD_Init();
  LCD_WriteComm( 0b00001111 ); // nastavi blikajici kurzor
  LCD_Position( 0, 0 );
  LCD_WriteCString("Nastaveni rele: ");
  LCD_Position( 1, 0 );
  LCD_WriteCString("R1_ R2_ R3_ R4_ ");
  KEYmin = 0;
  KEYmax = 3; // nastaveni rele
  KEYcounter = 0;
  for(i=0; i<4; i++)
  {
	LCD_Position( 1, 4 * i + 2 );
	if (((Rele>>i)&0x01)==0x01) LCD_WriteData( '^' );
	else LCD_WriteData( '_' );
  }
  while( KEYenter!=2 )
  {
	LCD_Position( 1, 4 * KEYcounter + 2 );
	Wait_ms( 300 ); 
	if(KEYenter==1)
	{
	  KEYenter = 0;
	  i = (Rele >> KEYcounter) & 0x01;
	  if(i==0)
	  {
	    switch(KEYcounter&0x03)
		{
		  case 0x00:
		  { 
		    Rele |= 0b00000001;
			SetPWM |= 0b00000011;
			SetPWM &= 0b11111011;
			break;
		  }
		  case 0x01:
		  {
		    Rele |= 0b00000010; 
			SetPWM |= 0b00110000;
			SetPWM &= 0b10111111;
			break;
		  }
		  case 0x02:
		  {
		    Rele |= 0b00000100;
			SetPWM |= 0b00110000;
			SetPWM &= 0b10111111;
			break;
		  }
		  case 0x03: Rele |= 0b00001000; break;
		}
		LCD_WriteData( '^' );
	  }
	  else 
	  {
	    switch(KEYcounter&0x03)
		{
		  case 0x00:
		  { 
		    Rele &= 0b11111110;
			SetPWM |= 0b00000001;
			SetPWM &= 0b11111001; 
			break;
		  }
		  case 0x01:
		  { 
		    Rele &= 0b11111101;
		    SetPWM |= 0b00010000;
			SetPWM &= 0b10011111;
			break;
		  }
		  case 0x02:
		  {
		    Rele &= 0b11111011;
		    SetPWM |= 0b00010000;
			SetPWM &= 0b10011111;
			break;
		  }
		  case 0x03: Rele &= 0b11110111; break;
		}
		LCD_WriteData( '_' );
      }
	}
  }
  KEYenter = 0;
}

unsigned char DS1302( unsigned char DS1302addr, unsigned char DS1302data )
{
  unsigned char i, u = 0;

  DS1302_ClrClk();
  DS1302_OutSda(); // vystup
  DS1302_ClrSda();
  DS1302_SetRst(); // obvod je vyresetovan
  DS1302addr |= 0x80; // nastavi posledni bit adresy
  for(i=0; i<8; i++) // bude posilat adresu
  {
    if((DS1302addr&0x01)==0x01) DS1302_SetSda()
	else DS1302_ClrSda();
	DS1302addr >>= 1;
	DS1302_SetClk();
	DS1302_ClrClk();
  }
  for(i=0; i<8; i++) // bude posilat data
  {
    if((DS1302data&0x01)==0x01) DS1302_SetSda()
	else DS1302_ClrSda();
	DS1302_OutSda(); // vystup
	DS1302_SetClk();
	DS1302_InpSda(); // vstup
	DS1302_ClrSda(); // pripojen pullup
	DS1302_ClrClk(); // konec pulsu hodin
	u >>= 1;
	DS1302data >>= 1; // posune bajt
	if((PINC&0x10)==0x10) u |= 0x80; // nastavi posledni bit
	else u &= ~0x80; // nuluje posledni bit
  }
  DS1302_ClrRst();
  if ((u&0x80)==0x80)
  {
    u <<= 1;
	u++;
  }
  else u <<= 1;
  return( u ); // vrati hodnotu prectenou z obvodu
}

ISR(INT1_vect) // podprogram preruseni INT1
{
  TCNT1H = 0x05;
  TCNT1L = 0x78; // posunuti o 1400 dilku

  Timer0++;
  if ((SetPWM&0x44)==0x00)
  {
    SET_Triak(); // zmena jasu je rychla
    SET_Triak();
    SET_Triak();
  }
  if(((SetPWM&0x44)!=0x00) && (Timer0==5)) SET_Triak(); // zmena jasu je pomala
  if(Timer0==5)
  {
    DS_Reset();
	DS_Write( 0xCC );
	DS_Write( 0xBE ); // prikaz ke cteni
  }
  if(Timer0==10) Teplota = DS_Read() + ( 0x100 * DS_Read() ); // vyctena teplota
  if(Timer0==15)
  {
  	DS_Reset();
	DS_Write( 0xCC );
	DS_Write( 0x44 ); // prikaz k mereni
	if(Teplota!=0xFFFF)
	{
	  if(Termostat>Teplota) Rele |= 0b00001000; // program termostatu *****
	  if(Teplota>Termostat) Rele &= 0b11110111; // vypina rele
	}
	else // neni pripojen teplomer
	{
	  Rele &= 0b11110111; // vypina rele
	}
  }
  if(Timer0==20) // provadi se kazde pate preruseni > 3x za sekundu
  {
    Timer0 = 0;
	Search_Timer(); // nastaveni rele podle casu v EEPROM
	LedTimer--;
	if(LedTimer==0x00) // ovladani podsviceni
	{
	  LCD_ClrLed();
	  LedTimer = 0x01;
	}
  }
  Rhod = DS1302( 0x05, 0x00 ); // ulozi hodiny
  Rmin = DS1302( 0x03, 0x00 ); // ulozi minuty
  Rsec = DS1302( 0x01, 0x00 ); // ulozi sekundy
  switch(PIND&0x07)
  {
    case 0b00000011: // pricte po 10ms stisku tlacitka
	{
	  if((KEYtimer==1) && (KEYcounter<KEYmax)) 
	  { 
	    if((KEYhex==0)&&((KEYcounter&0x0F)==0x09)) KEYcounter += 7;
		else KEYcounter++;
		KEYtimer1++;
	  }
	  KEYtimer++;
	  LCD_SetLed();
	  LedTimer = 0x80; // casovac nastaven na 20 sekund
	  break;
	}
	case 0b00000101: // odecte po 10ms stisku tlacitka
	{
	  if((KEYtimer==1) && (KEYcounter>KEYmin))
	  {
	    if((KEYhex==0)&&((KEYcounter&0x0F)==0x00)) KEYcounter -= 7;
		else KEYcounter--;
		KEYtimer1++;
	  }
	  KEYtimer++;
	  LCD_SetLed();
	  LedTimer = 0x80; // casovac nastaven na 20 sekund
	  break;
	}
	case 0b00000110: // stisknuta klavesa ENTER
	{
	  KEYenter1++;
	  if(KEYenter1==100) KEYenter1 = 0;
	  LCD_SetLed();
	  LedTimer = 0x80; // casovac nastaven na 20 sekund
	  break;
	}
	default:
	{
	  KEYtimer = 0;
	  KEYtimer1 = 0;
	  if(KEYenter1>4 ) KEYenter = 1;
	  if(KEYenter1>15) KEYenter = 2;
	  KEYenter1 = 0;
	  break;
	}
  }
  switch(KEYtimer1)
  {
    case 0x01: if(KEYtimer==20) KEYtimer = 0; break;
    case 0x02: if(KEYtimer==10) KEYtimer = 0; break;
	case 0x03: if(KEYtimer==5 ) KEYtimer = 0; break;
	default:   if(KEYtimer==2 ) KEYtimer = 0; break;
  }
}

void SYS_Init(void) // inicializace procesoru, rozsviti LED
{
  unsigned char i;

  PORTB = 0b00001000; // set Enable displej
  DDRB  = 0b11111111;
  PORTC = 0b00000100; // rozsviti displej
  DDRC  = 0b11101111; 
  PORTD = 0b00001111; // pripojen pullup
  DDRD  = 0b11110000; 
  TCCR1A = 0b11110010; // invertujici, PWM rezim 14
  TCCR1B = 0b00011010; // kmitocet je delen 8
  GICR   = 0b10000000; // povoleni preruseni INT1
  MCUCR  = 0b00001000; // aktivace preruseni sestupnou hranou 
  SREG  |= 0b10000000; // nastavi globalni preruseni
  ICR1H  = 0x26;
  ICR1L  = 0xB7; // hodnota k preteceni pro PWM modulaci triaku
  OCR1AH = 0x27;
  OCR1AL = 0x00; // triak je vypnut
  OCR1BH = 0x27;
  OCR1BL = 0x00; // triak je vypnut

  Rele = 0; // bity kopirovane na triaky
  Timer0 = 0; // casovac na 1/3 sekundy
  SetPWM = 0; // ukazuje stav PWM
  LCD_SetLed(); // rozsviti displej
  LedTimer = 50; // casovac nastaven na 4 sekundy
  Wait_ms( 200 );
  LCD_Init();
  Wait_ms( 200 );
  LCD_Init();
  LCD_Position( 0, 0 );
  LCD_WriteCString("System AKVA 0.3.");
  LCD_Position( 1, 0 );
  LCD_WriteCString("ATmega8         ");
  Wait_ms( 1000 );
  LCD_Init();

  KEYhex = 1; // hexadecimalni pocitani
  KEYtimer  = 0;
  KEYtimer1 = 0;
  KEYenter  = 0;
  KEYenter1 = 0; // promenne pro chod klavesnice v preruseni casovace 0

  DS1302_ClrRst();
  i = DS1302( 0b10010000, 0b10100101 ); // aktivace nabijeni
  i = DS1302( 0b10000001, 0b00000000 );
  i = DS1302( 0b10000000, i&0x7F ); // aktivace hodin

  Termostat = ((EEPROM_Read(0x80)) * 0x100) + EEPROM_Read(0x81); // nacte hodnotu termostatu z EEPROM
  if(Termostat > 1100) Termostat = 320; // 20 stupnu


  asm("wdr"); // nuluje watchdog
  WDTCR = 0x0F; // aktivuje Watchdog
}

unsigned char EEPROM_Read(unsigned char val)
{
  EEARL = val; EEARH = 0;
  EECR |= 0x01; // povoleni cteni z EEPROM
  return( EEDR );
}

void EEPROM_Write(unsigned char addr, unsigned char val)
{
  while(EECR&0x02); //ceka az EEWE bude 0
  EEARL = addr; EEARH = 0;
  EEDR = val;
  EECR |= 0x04;
  EECR |= 0x02;
}

void DS_Write( unsigned char value ) // Zapis bajtu na sbernici 1-wire, zapis trva 0.5ms
{ unsigned char i, temp;
  DS18_Out();
  for (i=0;i<8;i++)
  { 
    temp = value & 0x01;
	value >>= 1;
	DS18_Clr();
	Wait_us( 5 );
	if (temp==0) Wait_us( 30 );
	DS18_Set();
	Wait_us( 30 );
  }
}

unsigned char DS_Read( void ) // Cte bajt z obvodu Dallas cteni trva 0.5ms
{ 
  unsigned char i, value = 0;
  for (i=0;i<8;i++)
  { 
  	DS18_Out();
	DS18_Clr();
  	Wait_us( 2 );
  	DS18_In();
	Wait_us( 20 );
    if (PIND&0x10) value += 0x80;
	if (i < 7) value >>= 1;
    Wait_us( 30 );
  }
  return( value );
}

unsigned char DS_Reset(void) // Reset obvodu Dallas
{ 
  unsigned char value = 0;
  DS18_Out();
  DS18_Clr();
  Wait_us( 500 );		// 480us
  DS18_In();
  Wait_us( 70 );
  if (PIND&0x10) value++;
  Wait_us( 300 );
  if (!(PIND&0x10)) value++;
  return( value );
}   // vraci 0x00 - obvod je vyresetovan

