/*
==========================================
Name:		DS18B20.c
Author:		POSPISIL Frantisek
Created:	12/2008
Chip:		ATmega8
Frequency:	CPU 8MHz
Hardware:	DS18B20
==========================================
*/

/* hlavickove soubory */
#include <avr/io.h>
#include "DS18B20.h"
#include "Termostat.h"

/* definice funkci */
// ===========================================================================
void DS_Write( unsigned char value ) // Zapis bajtu na sbernici 1-wire
{ 
  unsigned char i, temp;

  SREG  &=~0x80; 	   // nuluje globalni preruseni
  for (i=0;i<8;i++)
  { 
    temp = value & 0x01;
	value >>= 1;
	DS18_Clr();
	DS18_Out();
	Wait_us( 5 );
	if (temp==0) Wait_us( 35 ); // 35
	DS18_In();
	DS18_Set();
	Wait_us( 40 ); // 40
  }
  SREG  |= 0x80; 	   // nastavi globalni preruseni
}

// ===========================================================================
unsigned char DS_Read( void ) // Cte bajt z obvodu Dallas
{ 
  unsigned char i, value = 0;

  SREG  &=~0x80; 	   // nuluje globalni preruseni
  for (i=0;i<8;i++)
  { 
	DS18_Clr();
	DS18_Out();
  	Wait_us( 3 ); // 3
  	DS18_In();
	Wait_us( 2 ); // 5
    if (PIND&0x80) value += 0x80;
	if (i < 7) value >>= 1;
    Wait_us( 40 ); // 40
  }
  SREG  |= 0x80; 	   // nastavi globalni preruseni
  return( value );
}

// ===========================================================================
unsigned char DS_Reset(void) // Reset obvodu Dallas
{ 
  unsigned char value = 0;

  SREG  &=~0x80; 	   // nuluje globalni preruseni
  DS18_Clr();
  DS18_Out();
  Wait_us( 600 );		// 480us
  DS18_In();
  Wait_us( 70 );
  if (PIND&0x80) value++;
  Wait_us( 300 );
  if (!(PIND&0x80)) value++;
  SREG  |= 0x80; 	   // nastavi globalni preruseni
  return( value );
}   // vraci 0x00 - obvod je vyresetovan
